/*
**  CXSC is a C++ library for eXtended Scientific Computing (V 2.2)
**
**  Copyright (C) 1990-2000 Institut fuer Angewandte Mathematik,
**                          Universitaet Karlsruhe, Germany
**            (C) 2000-2006 Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**  F. Blomquist, University of Wuppertal, 20.09.2007;
*/
/* CVS $Id: lx_complex.hpp,v 1.2 2007/05/01 10:09:53 cxsc Exp $ */

#ifndef LX_COMPLEX_HPP_L_INCLUDED
#define LX_COMPLEX_HPP_L_INCLUDED

#include <iostream>
#include <sstream>  // Wegen stringstream
#include <except.hpp>
#include <l_complex.hpp>
#include "lx_real.hpp"

namespace cxsc {

class lx_complex
{
private:
    // ------------- Data Elements -------------------------------------------
    lx_real re;
    lx_real im;

public:
    // ------------- Constructors --------------------------------------------

    lx_complex(void)  throw() {}
    lx_complex(int nr, const l_real &ar, int ni, const l_real &ai)
	throw() : re(lx_real(nr,ar)), im(lx_real(ni,ai))  { }
    lx_complex(int n, const real &a) 
	throw() : re(lx_real(n,a)), im(lx_real(0.0)) { }
    lx_complex(const lx_real &a, const lx_real &b)
	throw() : re(a), im(b) { }
    lx_complex(const lx_real &a) throw() : re(a), im(lx_real(0.0)) { }
    explicit lx_complex(const l_real &a, const l_real &b) 
	throw() : re(a), im(b) { }
    explicit lx_complex(const l_real &a) 
	throw() : re(a), im(lx_real(0.0)) { }
    explicit lx_complex(const real &a) 
	throw() : re(a), im(lx_real(0.0)) { }
    explicit lx_complex(const complex &a) 
	throw() : re(Re(a)), im(Im(a)) { }
    explicit lx_complex(const l_complex &a) 
	throw() : re(lx_real(0,Re(a))), im(lx_real(0,Im(a))) { }

// ------------- Assignments ---------------------------------------------

    lx_complex & operator = (const lx_real & lr) throw() 
        { re = lr; im = 0; return *this; }
    lx_complex & operator = (const l_real & lr) throw() 
        { re = lr; im = 0; return *this; }
    lx_complex & operator = (const real & lr) throw() 
        { re = lr; im = 0; return *this; }

    lx_complex & operator = (const l_complex & c) throw() 
        { re = Re(c); im = Im(c); return *this; }
    lx_complex & operator = (const complex & c) throw() 
        { re = Re(c); im = Im(c); return *this; } 
    
    // ------------- Functions -----------------------------------------------

    friend inline lx_real Re(const lx_complex&);
    friend inline lx_real Im(const lx_complex&);
    friend inline int StagPrec(const lx_complex&) throw();
    friend inline int expoRe(const lx_complex&) throw();
    friend inline int expoIm(const lx_complex&) throw();
    friend inline l_real lr_partRe(const lx_complex&) throw();
    friend inline l_real lr_partIm(const lx_complex&) throw();
 
    friend inline lx_complex & SetRe(lx_complex&, const lx_real&);
    friend inline lx_complex & SetRe(lx_complex&, const l_real&);
    friend inline lx_complex & SetRe(lx_complex&, const real&);

    friend inline lx_complex & SetIm(lx_complex&, const lx_real&);
    friend inline lx_complex & SetIm(lx_complex&, const l_real&);
    friend inline lx_complex & SetIm(lx_complex&, const real&);


    friend inline bool operator == (const lx_complex&, const lx_complex&) 
	throw();

    friend inline bool operator == (const lx_complex&, const l_complex&) 
	throw();
    friend inline bool operator == (const lx_complex&, const complex&) 
	throw();
    friend inline bool operator == (const l_complex&, const lx_complex&) 
	throw();
    friend inline bool operator == (const complex&,   const lx_complex&) 
	throw();

    friend inline bool operator == (const lx_complex&, const lx_real&) throw();
    friend inline bool operator == (const lx_complex&, const l_real&) throw();
    friend inline bool operator == (const lx_complex&, const real&)   throw();
    friend inline bool operator == (const lx_real&, const lx_complex&) throw();
    friend inline bool operator == (const l_real&, const lx_complex&) throw();
    friend inline bool operator == (const real&,   const lx_complex&) throw();

    friend inline lx_complex operator - (const lx_complex &) throw();


// ----------------------- Output --------------------------------------------

friend inline std::ostream& operator << (std::ostream& s, const lx_complex& a) 
    throw(); // A value a of type lx_complex is written to the output channel.

friend inline std::string & operator << (std::string &s, const lx_complex& a) 
    throw();
// The value of a variable a of type lx_complex is copied to a string s.
// s has the form:  { ? , ? }


}; // end of class lx_complex

// ***********************************************************************
// -----------------------------------------------------------------------
// ***********************************************************************

    inline std::ostream& operator << (std::ostream& s, const lx_complex&) 
	throw();
    inline std::string & operator << (std::string&  s, const lx_complex&) 
	throw(); 

// ------- friend functions declared in class lx_complex: -----------

    inline lx_real Re(const lx_complex&);
    inline lx_real Im(const lx_complex&);
    inline int StagPrec(const lx_complex&) throw();
    inline int expoRe(const lx_complex&) throw();
    inline int expoIm(const lx_complex&) throw();
    inline l_real lr_partRe(const lx_complex&) throw();
    inline l_real lr_partIm(const lx_complex&) throw();

    inline lx_complex & SetRe(lx_complex&, const lx_real&);
    inline lx_complex & SetRe(lx_complex&, const l_real&);
    inline lx_complex & SetRe(lx_complex&, const real&);

    inline lx_complex & SetIm(lx_complex&, const lx_real&);
    inline lx_complex & SetIm(lx_complex&, const l_real&);
    inline lx_complex & SetIm(lx_complex&, const real&);

    inline bool operator == (const lx_complex&, const lx_complex&) throw();

    inline bool operator == (const lx_complex&, const l_complex&) throw();
    inline bool operator == (const lx_complex&, const complex&) throw();
    inline bool operator == (const l_complex&, const lx_complex&) throw();
    inline bool operator == (const complex&,   const lx_complex&) throw();

    inline bool operator == (const lx_complex&, const lx_real&) throw();
    inline bool operator == (const lx_complex&, const l_real&) throw();
    inline bool operator == (const lx_complex&, const real&)   throw();
    inline bool operator == (const lx_real&, const lx_complex&) throw();
    inline bool operator == (const l_real&, const lx_complex&) throw();
    inline bool operator == (const real&,   const lx_complex&) throw();

// ----------------------------------------------------------------------

    inline bool operator != (const lx_complex&, const lx_complex&) throw();

    inline bool operator != (const lx_complex&, const l_complex&) throw();
    inline bool operator != (const lx_complex&, const complex&) throw();
    inline bool operator != (const l_complex&, const lx_complex&) throw();
    inline bool operator != (const complex&,   const lx_complex&) throw();

    inline bool operator != (const lx_complex&, const lx_real&) throw();
    inline bool operator != (const lx_complex&, const l_real&) throw();
    inline bool operator != (const lx_complex&, const real&)   throw();
    inline bool operator != (const lx_real&, const lx_complex&) throw();
    inline bool operator != (const l_real&, const lx_complex&) throw();
    inline bool operator != (const real&,   const lx_complex&) throw();

    inline lx_complex operator - (const lx_complex &) throw();

// -------------------------- Output ------------------------------------

    inline std::ostream& operator << (std::ostream& s, const lx_complex& a) 
    throw(); // A value a of type lx_complex is written to the output channel.
    inline std::string & operator << (std::string& s, const lx_complex& a) 
    throw();
    // The value of a variable a of type lx_complex is copied to a string s.
    // s has the form:  { ? , ? }

// ---- function declarations outside the class lx_complex ----

    inline lx_complex operator + (const lx_complex &) throw();
    

}  // end namespace cxsc

#include "lx_complex.inl"

#endif
