/*
**  CXSC is a C++ library for eXtended Scientific Computing (V 2.2)
**
**  Copyright (C) 1990-2000 Institut fuer Angewandte Mathematik,
**                          Universitaet Karlsruhe, Germany
**            (C) 2000-2006 Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**  F. Blomquist, University of Wuppertal, 20.09.2007;
*/
/* CVS $Id: lx_real.hpp,v 1.2 2007/05/01 10:09:53 cxsc Exp $ */

#ifndef LX_REAL_HPP_L_INCLUDED
#define LX_REAL_HPP_L_INCLUDED

#include <iostream>
#include <sstream>  // Wegen stringstream
#include <except.hpp>
#include <l_real.hpp>

namespace cxsc {

class lx_real
{
private:
    // ------------- Datenelemente -------------------------------------------
    int ex;
    l_real lr;

public:
    // ------------- Constructors --------------------------------------------

    lx_real(void)  throw() {}
    lx_real(int n, const l_real &a)   throw() : ex(n), lr(a) { }
    lx_real(int n, const real &a)     throw() : ex(n), lr(a) { }
    explicit lx_real(const l_real &a) throw() : ex(0), lr(a) { }
    explicit lx_real(const real &a)   throw() : ex(0), lr(a) { }

    // ------------- Assignments ---------------------------------------------

    inline lx_real & operator = (const lx_real &) throw();
    inline lx_real & operator = (const l_real &) throw();
    inline lx_real & operator = (const real &)   throw();

    // ------------- Functions -----------------------------------------------

    friend inline int StagPrec(const lx_real&) throw(); 
    friend inline int expo(const lx_real&) throw();
    friend inline int sign(const lx_real&) throw();
    friend inline l_real lr_part(const lx_real&) throw();
    friend inline lx_real abs(const lx_real&) throw();
    friend void scale_up  (lx_real &) throw();

    friend inline bool eq_zero (const lx_real &a) throw(); // a = 0;
    friend inline bool gr_zero (const lx_real &a) throw(); // a > 0;
    friend inline bool ge_zero (const lx_real &a) throw(); // a >=0;
    friend inline bool sm_zero (const lx_real &a) throw(); // a < 0;
    friend inline bool se_zero (const lx_real &a) throw(); // a <=0;

    friend inline lx_real operator -(const lx_real &) throw();

// ----------------------- Output --------------------------------------------

friend inline std::ostream& operator << (std::ostream& s, const lx_real& a) 
    throw(); // A value a of type lx_real is written to the output channel.

friend inline std::string & operator << (std::string &s, const lx_real& a) 
    throw();
// The value of a variable a of type lx_real is copied to a string s.
// s has the form:  {ex,li}



}; // end of class lx_real

// ------- friend functions declared in class lx_real: ---------

inline int StagPrec(const lx_real&) throw();
inline int expo(const lx_real&) throw();
inline int sign(const lx_real&) throw();
inline l_real lr_part(const lx_real&) throw();
inline lx_real abs(const lx_real&) throw();
       void scale_up  (lx_real&)  throw();

inline bool eq_zero (const lx_real &a) throw(); // a = 0;
inline bool gr_zero (const lx_real &a) throw(); // a > 0;
inline bool ge_zero (const lx_real &a) throw(); // a >=0;
inline bool sm_zero (const lx_real &a) throw(); // a < 0;
inline bool se_zero (const lx_real &a) throw(); // a <=0;

inline lx_real operator -(const lx_real &) throw();

// -------------------------- Output ------------------------------------

inline std::ostream& operator << (std::ostream& s, const lx_real& a) 
    throw(); // A value a of type lx_real is written to the output channel.
inline std::string & operator << (std::string &s, const lx_real& a) 
    throw();
// The value of a variable a of type lx_real is copied to a string s.
// s has the form:  {(ex),lr}

// ---- function declarations outside the class lx_real ----

inline lx_real operator +(const lx_real &) throw();

bool operator == (const lx_real &, const lx_real &) throw();
inline bool operator == (const lx_real &, const l_real &) throw();
inline bool operator == (const l_real &, const lx_real &) throw();
inline bool operator == (const lx_real &, const real &)   throw();
inline bool operator == (const real &,   const lx_real &) throw();

inline bool operator != (const lx_real &, const lx_real &) throw();
inline bool operator != (const lx_real &, const l_real &) throw();
inline bool operator != (const l_real &, const lx_real &) throw();
inline bool operator != (const lx_real &, const real &)   throw();
inline bool operator != (const real &,   const lx_real &) throw();

bool operator >  (const lx_real &, const lx_real &) throw();
inline bool operator <= (const lx_real &, const lx_real &) throw();
inline bool operator <  (const lx_real &, const lx_real &) throw();
inline bool operator >= (const lx_real &, const lx_real &) throw();

inline bool operator >  (const real &, const lx_real &) throw();
inline bool operator <= (const real &, const lx_real &) throw();
inline bool operator <  (const real &, const lx_real &) throw();
inline bool operator >= (const real &, const lx_real &) throw();

inline bool operator >  (const lx_real &, const real &) throw();
inline bool operator <= (const lx_real &, const real &) throw();
inline bool operator <  (const lx_real &, const real &) throw();
inline bool operator >= (const lx_real &, const real &) throw();

inline bool operator >  (const l_real &, const lx_real &) throw();
inline bool operator <= (const l_real &, const lx_real &) throw();
inline bool operator <  (const l_real &, const lx_real &) throw();
inline bool operator >= (const l_real &, const lx_real &) throw();

inline bool operator >  (const lx_real &, const l_real &) throw();
inline bool operator <= (const lx_real &, const l_real &) throw();
inline bool operator <  (const lx_real &, const l_real &) throw();
inline bool operator >= (const lx_real &, const l_real &) throw();

inline int add_int(int,int); // int addition    with overflow handling
inline int sub_int(int,int); // int subtraction with overflow handling

inline lx_real max(const lx_real&, const lx_real&);
inline lx_real min(const lx_real&, const lx_real&);

}  // end namespace cxsc

#include "lx_real.inl"

#endif
